/**
 * Copyright (C) 2010-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.rackspace;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Locale;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.dc.DataCenter;
import org.dasein.cloud.dc.DataCenterServices;
import org.dasein.cloud.dc.Region;

public class RackspaceDataCenter implements DataCenterServices {

    private RackspaceCloud rackspace;
    
    public RackspaceDataCenter(RackspaceCloud provider) { rackspace = provider; }
    
    @Override
    public DataCenter getDataCenter(String dataCenterId) throws InternalException, CloudException {
        String regionId = rackspace.getContext().getRegionId();
        
        if( regionId == null ) {
            regionId = "main";
        }
        if( dataCenterId.equals("1") && regionId.equals("main") ) {
            DataCenter dataCenter = new DataCenter();
            
            dataCenter.setActive(true);
            dataCenter.setAvailable(true);
            dataCenter.setName("DFW or ORD");
            dataCenter.setProviderDataCenterId("1");
            dataCenter.setRegionId("main");
            return dataCenter;
        }
        else if( dataCenterId.equals("lon1") && regionId.equals("uk") ) {
            DataCenter dataCenter = new DataCenter();
            
            dataCenter.setActive(true);
            dataCenter.setAvailable(true);
            dataCenter.setName("London 1");
            dataCenter.setProviderDataCenterId("lon1");
            dataCenter.setRegionId("uk");
            return dataCenter;            
        }
        return null;
    }

    public String getProviderTermForDataCenter(Locale locale) {
        return "data center";
    }

    public String getProviderTermForRegion(Locale locale) {
        return "region";
    }

    public Region getRegion(String regionId) throws InternalException, CloudException {
        if( regionId.equals("main") ) {
            Region region = new Region();
            
            region.setName("United States");
            region.setProviderRegionId(regionId);
            region.setAvailable(true);
            region.setActive(true);
            return region;
        }
        else if( regionId.equals("uk") ) {
            Region region = new Region();
            
            region.setName("United Kingdom");
            region.setProviderRegionId(regionId);
            region.setAvailable(true);
            region.setActive(true);
            return region;
        }
        return null;
    }

    public Collection<DataCenter> listDataCenters(String regionId) throws InternalException, CloudException {
        if( regionId != null ) {
            if( regionId.equals("main") ) {
                return Collections.singletonList(getDataCenter("1"));
            }
            else if( regionId.equals("uk") ) {
                return Collections.singletonList(getDataCenter("lon1"));
            }
        }
        return Collections.emptyList();
    }

    public Collection<Region> listRegions() throws InternalException, CloudException {
        ArrayList<Region> regions = new ArrayList<Region>();
        String endpoint = rackspace.getContext().getEndpoint();

        if( endpoint == null || endpoint.trim().equals("") || endpoint.equals("https://auth.api.rackspacecloud.com/v1.0") ) {
            regions.add(getRegion("main"));
        }
        else {
            regions.add(getRegion("uk"));
        }
        return regions;
    }

    public String matchDataCenter(CloudProvider provider, String dataCenterId) throws InternalException, CloudException {
        return dataCenterId;
    }

}
