/**
 * Copyright (C) 2010-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.rackspace.network;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Locale;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.identity.ServiceAction;
import org.dasein.cloud.jclouds.rackspace.RackspaceCloud;
import org.dasein.cloud.network.LbAlgorithm;
import org.dasein.cloud.network.LbListener;
import org.dasein.cloud.network.LbProtocol;
import org.dasein.cloud.network.LoadBalancer;
import org.dasein.cloud.network.LoadBalancerAddressType;
import org.dasein.cloud.network.LoadBalancerState;
import org.dasein.cloud.network.LoadBalancerSupport;
import org.jclouds.cloudloadbalancers.CloudLoadBalancersAsyncClient;
import org.jclouds.cloudloadbalancers.CloudLoadBalancersClient;
import org.jclouds.cloudloadbalancers.domain.LoadBalancer.Status;
import org.jclouds.cloudloadbalancers.domain.Node;
import org.jclouds.cloudloadbalancers.domain.VirtualIP;
import org.jclouds.rest.RestContext;

public class RackspaceLoadBalancer implements LoadBalancerSupport {
    private RackspaceCloud provider;
    
    RackspaceLoadBalancer(RackspaceCloud provider) { this.provider = provider; }
    
    @Override
    public void addDataCenters(String toLoadBalancerId, String... dataCenterIdsToAdd) throws CloudException, InternalException {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void addServers(String toLoadBalancerId, String... serverIdsToAdd) throws CloudException, InternalException {
        // TODO Auto-generated method stub
        
    }

    @Override
    public String create(String name, String description, String addressId, String[] dataCenterIds, LbListener[] listeners, String[] serverIds) throws CloudException, InternalException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public LoadBalancer getLoadBalancer(String loadBalancerId) throws CloudException, InternalException {
        RestContext<CloudLoadBalancersClient,CloudLoadBalancersAsyncClient> ctx = provider.getLoadBalancersContext();
        CloudLoadBalancersClient client = ctx.getApi();
        try {
            for( String region : client.getConfiguredRegions() ) {
                org.jclouds.cloudloadbalancers.domain.LoadBalancer lb = client.getLoadBalancerClient(region).getLoadBalancer(Integer.parseInt(loadBalancerId));
                
                if( lb != null ) {
                    LoadBalancer dslb = toLoadBalancer(lb);
                    
                    if( dslb != null ) {
                        return dslb;
                    }
                }
            }
            return null;
        }
        finally {
            ctx.close();
        }
    }

    @Override
    public LoadBalancerAddressType getAddressType() throws CloudException, InternalException {
        return LoadBalancerAddressType.IP;
    }

    @Override
    public int getMaxPublicPorts() throws CloudException, InternalException {
        return 0;
    }

    @Override
    public String getProviderTermForLoadBalancer(Locale locale) {
        return "load balancer";
    }

    @Override
    public Iterable<LbAlgorithm> listSupportedAlgorithms() throws CloudException, InternalException {
        return Collections.singletonList(LbAlgorithm.ROUND_ROBIN);
    }

    @Override
    public Iterable<LbProtocol> listSupportedProtocols() throws CloudException, InternalException {
        return Collections.singletonList(LbProtocol.RAW_TCP);
    }

    @Override
    public boolean isAddressAssignedByProvider() throws CloudException, InternalException {
        return false;
    }

    @Override
    public boolean isDataCenterLimited() throws CloudException, InternalException {
        return false;
    }

    @Override
    public boolean requiresListenerOnCreate() throws CloudException, InternalException {
        return false;
    }

    @Override
    public boolean requiresServerOnCreate() throws CloudException, InternalException {
        return false;
    }

    @Override
    public boolean isSubscribed() throws CloudException, InternalException {
        return provider.getContext().getRegionId().equals("main");
    }

    @Override
    public boolean supportsMonitoring() throws CloudException, InternalException {
        return false;
    }

    @Override
    public Iterable<LoadBalancer> listLoadBalancers() throws CloudException, InternalException {
        RestContext<CloudLoadBalancersClient,CloudLoadBalancersAsyncClient> ctx = provider.getLoadBalancersContext();
        CloudLoadBalancersClient client = ctx.getApi();
        try {
            ArrayList<LoadBalancer> loadBalancers = new ArrayList<LoadBalancer>();
            
            for( String region : client.getConfiguredRegions() ) {
                for( org.jclouds.cloudloadbalancers.domain.LoadBalancer lb : client.getLoadBalancerClient(region).listLoadBalancers() ) {
                    LoadBalancer dslb = toLoadBalancer(lb);
                    
                    if( dslb != null ) {
                        loadBalancers.add(dslb);
                    }
                }
            }
            return loadBalancers;
        }
        finally {
            ctx.close();
        }
    }
    
	@Override
	public String[] mapServiceAction(ServiceAction action) {
		return new String[0];
	}
    
    @Override
    public void remove(String loadBalancerId) throws CloudException, InternalException {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void removeDataCenters(String fromLoadBalancerId, String... dataCenterIdsToRemove) throws CloudException, InternalException {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void removeServers(String fromLoadBalancerId, String... serverIdsToRemove) throws CloudException, InternalException {
        // TODO Auto-generated method stub
        
    }

    private LbAlgorithm toAlgorithm(String algorithm) {
        System.out.println("Algorithm=" + algorithm);
        return LbAlgorithm.ROUND_ROBIN;
    }
    
    private LoadBalancer toLoadBalancer(org.jclouds.cloudloadbalancers.domain.LoadBalancer lb) {
        LoadBalancer loadBalancer = new LoadBalancer();
        
        loadBalancer.setProviderLoadBalancerId(String.valueOf(lb.getId()));
        loadBalancer.setCreationTimestamp(lb.getCreated().getTime());
        loadBalancer.setName(lb.getName());
        loadBalancer.setDescription(lb.getName());
        loadBalancer.setProviderOwnerId(provider.getContext().getAccountNumber());
        for( VirtualIP ip : lb.getVirtualIPs() ) {
            loadBalancer.setAddress(ip.getAddress());
        }
        loadBalancer.setAddressType(LoadBalancerAddressType.IP);
        loadBalancer.setCurrentState(toState(lb.getStatus()));
        loadBalancer.setProviderRegionId(provider.getContext().getRegionId());
        ArrayList<String> providerServerIds = new ArrayList<String>();
        
        for( Node node : lb.getNodes() ) {
            providerServerIds.add(String.valueOf(node.getId()));
        }
        loadBalancer.setProviderServerIds(providerServerIds.toArray(new String[providerServerIds.size()]));
        ArrayList<LbListener> listeners = new ArrayList<LbListener>();
        
        LbListener listener = new LbListener();
            
        listener.setNetworkProtocol(toProtocol(lb.getProtocol()));
        listener.setPrivatePort(lb.getPort() == null ? -1 : lb.getPort());
        listener.setPublicPort(listener.getPrivatePort());
        listener.setAlgorithm(toAlgorithm(lb.getAlgorithm()));
        listeners.add(listener);
        loadBalancer.setListeners(listeners.toArray(new LbListener[listeners.size()]));
        return loadBalancer;
    }

    private LbProtocol toProtocol(String protocol) {
        if( protocol == null || protocol.equalsIgnoreCase("tcp") ) {
            return LbProtocol.RAW_TCP;
        }
        else if( protocol.equalsIgnoreCase("http") ) {
            return LbProtocol.HTTP;
        }
        else if( protocol.equalsIgnoreCase("https") ) {
            return LbProtocol.HTTPS;
        }
        else if( protocol.equalsIgnoreCase("ajb") ) {
            return LbProtocol.AJP;
        }
        else {
            System.out.println("Unknown protocol: " + protocol);
            return LbProtocol.RAW_TCP;
        }
    }

    private LoadBalancerState toState(Status status) {
        if( status.equals(Status.ACTIVE) ) {
            return LoadBalancerState.ACTIVE;
        }
        else if( status.equals(Status.DELETED) ) {
            return LoadBalancerState.TERMINATED;
        }
        return LoadBalancerState.PENDING;
    }
}
